from distutils.core import setup

setup(
    name='schemato',
    version='1.0',
    py_modules=['schemato'],

    author='emmett butler',
    author_email='emmett.butler321@gmail.com',
    url='http://github.com/emmett9001/schemato',
    description='Unified semantic metadata validator'
)
